diff --git a/web/pgadmin/tools/sqleditor/tests/test_encoding_charset.py b/web/pgadmin/tools/sqleditor/tests/test_encoding_charset.py
index 8de9ee6e..ef565cb9 100644
--- a/web/pgadmin/tools/sqleditor/tests/test_encoding_charset.py
+++ b/web/pgadmin/tools/sqleditor/tests/test_encoding_charset.py
@@ -50,14 +50,14 @@ class TestEncodingCharset(BaseTestGenerator):
             dict(
                 db_encoding='SQL_ASCII',
                 lc_collate='C',
-                test_str='\\255'
+                test_str='Tif'
             )),
         (
             'With Encoding LATIN1',
             dict(
                 db_encoding='LATIN1',
                 lc_collate='C',
-                test_str='Ň'
+                test_str='äöüßÑ'
             )),
         (
             'With Encoding LATIN2',
@@ -66,7 +66,27 @@ class TestEncodingCharset(BaseTestGenerator):
                 lc_collate='C',
                 test_str='§'
             )),
-
+        (
+            'With Encoding LATIN9',
+            dict(
+                db_encoding='LATIN9',
+                lc_collate='C',
+                test_str='äöüß'
+            )),
+        (
+            'With Encoding EUC_JIS_2004',
+            dict(
+                db_encoding='EUC_JIS_2004',
+                lc_collate='C',
+                test_str='じんぼはりんごをたべる'
+            )),
+        (
+            'With Encoding WIN1256',
+            dict(
+                db_encoding='WIN1256',
+                lc_collate='C',
+                test_str='صباح الخير'
+            )),
     ]
 
     def setUp(self):
@@ -113,6 +133,11 @@ class TestEncodingCharset(BaseTestGenerator):
         self.assertEquals(response.status_code, 200)
         response_data = json.loads(response.data.decode('utf-8'))
         self.assertEquals(response_data['data']['rows_fetched_to'], 1)
+        if (type(response_data['data']['result'][0][0])) == unicode:
+            result = response_data['data']['result'][0][0].encode('utf-8')
+        else:
+            result = response_data['data']['result'][0][0]
+        self.assertEquals(result, self.test_str)
 
         database_utils.disconnect_database(self, self.encode_sid,
                                            self.encode_did)
diff --git a/web/pgadmin/utils/driver/psycopg2/connection.py b/web/pgadmin/utils/driver/psycopg2/connection.py
index 012266d2..be60bd2e 100644
--- a/web/pgadmin/utils/driver/psycopg2/connection.py
+++ b/web/pgadmin/utils/driver/psycopg2/connection.py
@@ -400,7 +400,7 @@ class Connection(BaseConnection):
         if self.use_binary_placeholder:
             register_binary_typecasters(self.conn)
 
-        postgres_encoding, self.python_encoding = \
+        postgres_encoding, self.python_encoding, typecast_encoding = \
             getEncoding(self.conn.encoding)
 
         # Note that we use 'UPDATE pg_settings' for setting bytea_output as a
@@ -647,11 +647,7 @@ WHERE
             params: Extra parameters
         """
 
-        if sys.version_info < (3,):
-            if type(query) == unicode:
-                query = query.encode('utf-8')
-        else:
-            query = query.encode('utf-8')
+        query = query.encode(self.python_encoding)
 
         params = self.escape_params_sqlascii(params)
         cur.execute(query, params)
@@ -680,16 +676,13 @@ WHERE
             return False, str(cur)
         query_id = random.randint(1, 9999999)
 
-        if IS_PY2 and type(query) == unicode:
-            query = query.encode('utf-8')
-
         current_app.logger.log(
             25,
             u"Execute (with server cursor) for server #{server_id} - "
             u"{conn_id} (Query-id: {query_id}):\n{query}".format(
                 server_id=self.manager.sid,
                 conn_id=self.conn_id,
-                query=query.decode('utf-8') if
+                query=query.decode(self.python_encoding) if
                 sys.version_info < (3,) else query,
                 query_id=query_id
             )
@@ -943,11 +936,9 @@ WHERE
             formatted exception message
         """
 
-        if sys.version_info < (3,):
-            if type(query) == unicode:
-                query = query.encode('utf-8')
-        else:
-            query = query.encode('utf-8')
+        encoding = self.python_encoding
+
+        query = query.encode(encoding)
 
         # Convert the params based on python_encoding
         params = self.escape_params_sqlascii(params)
@@ -965,7 +956,7 @@ WHERE
             u"{query_id}):\n{query}".format(
                 server_id=self.manager.sid,
                 conn_id=self.conn_id,
-                query=query.decode('utf-8'),
+                query=query.decode(encoding),
                 query_id=query_id
             )
         )
@@ -984,7 +975,7 @@ WHERE
                 u"Error Message:{errmsg}".format(
                     server_id=self.manager.sid,
                     conn_id=self.conn_id,
-                    query=query.decode('utf-8'),
+                    query=query.decode(encoding),
                     errmsg=errmsg,
                     query_id=query_id
                 )
diff --git a/web/pgadmin/utils/driver/psycopg2/encoding.py b/web/pgadmin/utils/driver/psycopg2/encoding.py
index 1f4a04f7..8ee648c4 100644
--- a/web/pgadmin/utils/driver/psycopg2/encoding.py
+++ b/web/pgadmin/utils/driver/psycopg2/encoding.py
@@ -10,20 +10,23 @@
 #  Get Postgres and Python encoding
 
 encode_dict = {
-    'SQL_ASCII': ['SQL_ASCII', 'raw_unicode_escape'],
-    'SQLASCII': ['SQL_ASCII', 'raw_unicode_escape'],
-    'MULE_INTERNAL': ['MULE_INTERNAL', 'raw_unicode_escape'],
-    'MULEINTERNAL': ['MULEINTERNAL', 'raw_unicode_escape'],
-    'LATIN1': ['LATIN1', 'latin1'],
-    'LATIN2': ['LATIN2', 'latin2'],
-    'LATIN3': ['LATIN3', 'latin3'],
-    'LATIN4': ['LATIN4', 'latin4'],
-    'LATIN5': ['LATIN5', 'latin5'],
-    'LATIN6': ['LATIN6', 'latin6'],
-    'LATIN7': ['LATIN7', 'latin7'],
-    'LATIN8': ['LATIN8', 'latin8'],
-    'LATIN9': ['LATIN9', 'latin9'],
-    'LATIN10': ['LATIN10', 'latin10']
+    'SQL_ASCII': ['SQL_ASCII', 'raw_unicode_escape', 'unicode_escape'],
+    'SQLASCII': ['SQL_ASCII', 'raw_unicode_escape', 'unicode_escape'],
+    'MULE_INTERNAL': ['MULE_INTERNAL', 'raw_unicode_escape', 'unicode_escape'],
+    'MULEINTERNAL': ['MULEINTERNAL', 'raw_unicode_escape', 'unicode_escape'],
+    'LATIN1': ['LATIN1', 'latin1', 'latin1'],
+    'LATIN2': ['LATIN2', 'latin2', 'latin2'],
+    'LATIN3': ['LATIN3', 'latin3', 'latin3'],
+    'LATIN4': ['LATIN4', 'latin4', 'latin4'],
+    'LATIN5': ['LATIN5', 'latin5', 'latin5'],
+    'LATIN6': ['LATIN6', 'latin6', 'latin6'],
+    'LATIN7': ['LATIN7', 'latin7', 'latin7'],
+    'LATIN8': ['LATIN8', 'latin8', 'latin8'],
+    'LATIN9': ['LATIN9', 'latin9', 'latin9'],
+    'LATIN10': ['LATIN10', 'latin10', 'latin10'],
+    'EUC_JIS_2004': ['UTF8', 'utf-8', 'utf-8'],
+    'EUCJIS2004': ['UTF8', 'utf-8', 'utf-8'],
+    'WIN1256': ['UTF8', 'utf-8', 'utf-8']
 }
 
 
@@ -33,7 +36,7 @@ def getEncoding(key):
     :return:
     [Postgres_encoding, Python_encoding] - Postgres and Python encoding
     """
-    return encode_dict.get(key, ['UNICODE', 'utf-8'])
+    return encode_dict.get(key, ['UNICODE', 'utf-8', 'utf-8'])
 
 
 def configureDriverEncodings(encodings):
@@ -43,5 +46,5 @@ def configureDriverEncodings(encodings):
     # because for parameterized DML, param values are converted based on
     # python encoding of pyscopg2s internal encodings dict.
     for key, val in encode_dict.items():
-        postgres_encoding, python_encoding = val
+        postgres_encoding, python_encoding, typecast_encoding = val
         encodings[key] = python_encoding
diff --git a/web/pgadmin/utils/driver/psycopg2/typecast.py b/web/pgadmin/utils/driver/psycopg2/typecast.py
index 1f462f08..1aa7089b 100644
--- a/web/pgadmin/utils/driver/psycopg2/typecast.py
+++ b/web/pgadmin/utils/driver/psycopg2/typecast.py
@@ -19,7 +19,7 @@ import psycopg2
 from psycopg2.extensions import encodings
 from psycopg2.extras import Json as psycopg2_json
 
-from .encoding import configureDriverEncodings
+from .encoding import configureDriverEncodings, getEncoding
 
 configureDriverEncodings(encodings)
 
@@ -181,20 +181,22 @@ def register_string_typecasters(connection):
     # characters. Here we unescape them using unicode_escape
     # and send ahead. When insert update is done, the characters
     # are escaped again and sent to the DB.
-    if connection.encoding in ('SQL_ASCII', 'SQLASCII',
-                               'MULE_INTERNAL', 'MULEINTERNAL'):
+
+    postgres_encoding, python_encoding, typecast_encoding = \
+        getEncoding(connection.encoding)
+    if postgres_encoding != 'UNICODE':
         if sys.version_info >= (3,):
             def non_ascii_escape(value, cursor):
                 if value is None:
                     return None
                 return bytes(
                     value, encodings[cursor.connection.encoding]
-                ).decode('unicode_escape', errors='replace')
+                ).decode(typecast_encoding, errors='replace')
         else:
             def non_ascii_escape(value, cursor):
                 if value is None:
                     return None
-                return value.decode('unicode_escape', errors='replace')
+                return value.decode(typecast_encoding, errors='replace')
                 # return value
 
         unicode_type = psycopg2.extensions.new_type(
diff --git a/web/pgadmin/utils/tests/test_encoding.py b/web/pgadmin/utils/tests/test_encoding.py
index e625fd0c..2ac19dc9 100644
--- a/web/pgadmin/utils/tests/test_encoding.py
+++ b/web/pgadmin/utils/tests/test_encoding.py
@@ -16,79 +16,87 @@ class TestEncoding(BaseTestGenerator):
             'When the database encoding is SQL_ASCII',
             dict(
                 db_encoding='SQL_ASCII',
-                expected_return_value=['SQL_ASCII', 'raw_unicode_escape']
+                expected_return_value=['SQL_ASCII', 'raw_unicode_escape',
+                                       'unicode_escape']
             )
         ), (
             'When the database encoding is MULEINTERNAL',
             dict(
                 db_encoding='MULEINTERNAL',
-                expected_return_value=['MULEINTERNAL', 'raw_unicode_escape']
+                expected_return_value=['MULEINTERNAL', 'raw_unicode_escape',
+                                       'unicode_escape']
             )
         ), (
             'When the database encoding is LATIN1',
             dict(
                 db_encoding='LATIN1',
-                expected_return_value=['LATIN1', 'latin1']
+                expected_return_value=['LATIN1', 'latin1', 'latin1']
             )
         ), (
             'When the database encoding is LATIN2',
             dict(
                 db_encoding='LATIN2',
-                expected_return_value=['LATIN2', 'latin2']
+                expected_return_value=['LATIN2', 'latin2', 'latin2']
             )
         ), (
             'When the database encoding is LATIN3',
             dict(
                 db_encoding='LATIN3',
-                expected_return_value=['LATIN3', 'latin3']
+                expected_return_value=['LATIN3', 'latin3', 'latin3']
             )
         ), (
             'When the database encoding is LATIN4',
             dict(
                 db_encoding='LATIN4',
-                expected_return_value=['LATIN4', 'latin4']
+                expected_return_value=['LATIN4', 'latin4', 'latin4']
             )
         ), (
             'When the database encoding is LATIN5',
             dict(
                 db_encoding='LATIN5',
-                expected_return_value=['LATIN5', 'latin5']
+                expected_return_value=['LATIN5', 'latin5', 'latin5']
             )
         ), (
             'When the database encoding is LATIN6',
             dict(
                 db_encoding='LATIN6',
-                expected_return_value=['LATIN6', 'latin6']
+                expected_return_value=['LATIN6', 'latin6', 'latin6']
             )
         ), (
             'When the database encoding is LATIN7',
             dict(
                 db_encoding='LATIN7',
-                expected_return_value=['LATIN7', 'latin7']
+                expected_return_value=['LATIN7', 'latin7', 'latin7']
             )
         ), (
             'When the database encoding is LATIN8',
             dict(
                 db_encoding='LATIN8',
-                expected_return_value=['LATIN8', 'latin8']
+                expected_return_value=['LATIN8', 'latin8', 'latin8']
             )
         ), (
             'When the database encoding is LATIN9',
             dict(
                 db_encoding='LATIN9',
-                expected_return_value=['LATIN9', 'latin9']
+                expected_return_value=['LATIN9', 'latin9', 'latin9']
             )
         ), (
             'When the database encoding is LATIN10',
             dict(
                 db_encoding='LATIN10',
-                expected_return_value=['LATIN10', 'latin10']
+                expected_return_value=['LATIN10', 'latin10', 'latin10']
             )
         ), (
             'When the database encoding is WIN1258',
             dict(
                 db_encoding='WIN1258',
-                expected_return_value=['UNICODE', 'utf-8']
+                expected_return_value=['UNICODE', 'utf-8', 'utf-8']
+            )
+        ), (
+            'When the database encoding is WIN1256',
+            dict(
+                db_encoding='WIN1256',
+                expected_return_value=['UTF8', 'utf-8', 'utf-8']
             )
         ),
     ]
